import React, { ReactNode, useCallback, useEffect, Suspense } from "react";
import { BrowserRouter, Routes, Route, Navigate } from "react-router-dom";
import { route, navigate, operation, elements } from "@/router";
import { observer } from "mobx-react-lite";
import { useStores } from "@/store/index";
import Loading from "@/views/Loading";
import { current, menu } from "@/api/demo_api";
import localStorageKey from "@/utils/localStorageKey";

const MyRouter: React.FC<{}> = observer(() => {
  const { permissionStore, menuStore } = useStores();

  useEffect(() => {
    current().then((res) => {
      localStorage.setItem(localStorageKey.USER_INFO, JSON.stringify(res));
      menu().then((res) => {
        permissionStore.restAddRoutes();
        for (let item of res.data) {
          for (let route of item.routes) {
            route.element = elements[route.element];
            route.path = `/product/${item.id}${route.path}`;
          }
          permissionStore.setAddRoutes(item.routes);
        }
        menuStore.initMenu(res.data);
        permissionStore.initAllRoutes();
      });
    });
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const setRoutes = useCallback(
    (routes: Array<route | navigate | operation>): ReactNode => {
      return routes.map((item, index) => {
        if (item.type === "page") {
          if (item.children && item.children.length) {
            return (
              <Route
                key={item.path}
                path={item.path}
                element={
                  <Suspense fallback={<Loading></Loading>}>
                    <item.element childrenRoutes={item.children} />
                  </Suspense>
                }
              >
                {setRoutes(item.children)}
              </Route>
            );
          } else {
            return (
              <Route
                key={item.path}
                path={item.path}
                element={
                  <Suspense fallback={<Loading></Loading>}>
                    <item.element />
                  </Suspense>
                }
              ></Route>
            );
          }
        } else if (item.type === "navigate") {
          return (
            <Route
              key={item.from}
              path={item.from}
              element={<Navigate to={item.to}></Navigate>}
            ></Route>
          );
        } else {
          return null;
        }
      });
    },
    []
  );

  return (
    <BrowserRouter>
      <Routes>{setRoutes(permissionStore.allRoutes)}</Routes>
    </BrowserRouter>
  );
});

export default MyRouter;
